/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import { FC, useEffect, useState } from 'react';
import { useTranslation } from 'react-i18next';

import { marked } from 'marked';

import type * as Type from '@/common/interface';
import { SchemaForm, JSONSchema, initFormData, UISchema } from '@/components';
import { useToast } from '@/hooks';
import { getLegalSetting, putLegalSetting } from '@/services';
import { handleFormError, scrollToElementTop } from '@/utils';
import { siteLealStore } from '@/stores';

const Legal: FC = () => {
  const { t } = useTranslation('translation', {
    keyPrefix: 'admin.legal',
  });
  const Toast = useToast();

  const externalContent = [
    {
      value: 'always_display',
      label: t('external_content_display.always_display'),
    },
    {
      value: 'ask_before_display',
      label: t('external_content_display.ask_before_display'),
    },
  ];

  const schema: JSONSchema = {
    title: t('page_title'),
    required: ['terms_of_service', 'privacy_policy'],
    properties: {
      external_content_display: {
        type: 'string',
        title: t('external_content_display.label'),
        description: t('external_content_display.text'),
        enum: externalContent?.map((lang) => lang.value),
        enumNames: externalContent?.map((lang) => lang.label),
        default: 0,
      },
      terms_of_service: {
        type: 'string',
        title: t('terms_of_service.label'),
        description: t('terms_of_service.text'),
      },
      privacy_policy: {
        type: 'string',
        title: t('privacy_policy.label'),
        description: t('privacy_policy.text'),
      },
    },
  };
  const uiSchema: UISchema = {
    external_content_display: {
      'ui:widget': 'select',
    },
    terms_of_service: {
      'ui:widget': 'textarea',
      'ui:options': {
        rows: 10,
      },
    },
    privacy_policy: {
      'ui:widget': 'textarea',
      'ui:options': {
        rows: 10,
      },
    },
  };
  const [formData, setFormData] = useState(initFormData(schema));

  const onSubmit = (evt) => {
    evt.preventDefault();
    evt.stopPropagation();

    const reqParams: Type.AdminSettingsLegal = {
      external_content_display: formData.external_content_display.value,
      terms_of_service_original_text: formData.terms_of_service.value,
      terms_of_service_parsed_text: marked.parse(
        formData.terms_of_service.value,
      ),
      privacy_policy_original_text: formData.privacy_policy.value,
      privacy_policy_parsed_text: marked.parse(formData.privacy_policy.value),
    };

    putLegalSetting(reqParams)
      .then(() => {
        Toast.onShow({
          msg: t('update', { keyPrefix: 'toast' }),
          variant: 'success',
        });
        siteLealStore.getState().update({
          external_content_display: reqParams.external_content_display,
        });
      })
      .catch((err) => {
        if (err.isError) {
          const data = handleFormError(err, formData);
          setFormData({ ...data });
          const ele = document.getElementById(err.list[0].error_field);
          scrollToElementTop(ele);
        }
      });
  };

  useEffect(() => {
    getLegalSetting().then((setting) => {
      if (setting) {
        const formMeta = { ...formData };
        formMeta.external_content_display.value =
          setting.external_content_display;
        formMeta.terms_of_service.value =
          setting.terms_of_service_original_text;
        formMeta.privacy_policy.value = setting.privacy_policy_original_text;
        setFormData(formMeta);
      }
    });
  }, []);

  const handleOnChange = (data) => {
    setFormData(data);
  };

  return (
    <>
      <h3 className="mb-4">{t('page_title')}</h3>
      <SchemaForm
        schema={schema}
        formData={formData}
        onSubmit={onSubmit}
        uiSchema={uiSchema}
        onChange={handleOnChange}
      />
    </>
  );
};

export default Legal;
